@@ -10,35 +10,44 @@ This script is designed to be downloaded and run independently, and
 will guide the process of downloading and installing all other source
 code.
 
-(c) 2009: Orion Buske <stasis@u.washington.edu>
+(c) 2009: Orion Buske <stasis {at} uw {dot} edu>
 
 """
-MIN_SEGTOOLS_VERSION = "0.1.1"
+PKG_VERSION = "0.1.2"
 
 ####################### BEGIN COMMON CODE HEADER #####################
 
+## Known issue: cannot deal with installing setuptools into the
+##   lib directory of the python installation (site.py conflict)
+
 import os
-import pkg_resources
 import sys
 
 from distutils.spawn import find_executable
 from distutils.version import LooseVersion
-from urllib import urlretrieve
 from site import addsitedir
 from string import Template
 from subprocess import call, PIPE, Popen
 from tempfile import gettempdir
+from urllib import urlretrieve
 
 assert sys.version_info >= (2, 4)
 
 MIN_HDF5_VERSION = "1.8"
 MIN_NUMPY_VERSION = "1.2"
 
-HDF5_URL = "http://www.hdfgroup.org/ftp/HDF5/prev-releases/hdf5-1.8.2/src/hdf5-1.8.2.tar.gz"
+HDF5_URL = "http://www.hdfgroup.org/ftp/HDF5/prev-releases/hdf5-1.8.2/" \
+    "src/hdf5-1.8.2.tar.gz"
 EZ_SETUP_URL = "http://peak.telecommunity.com/dist/ez_setup.py"
 
 EASY_INSTALL_PROMPT = "May I install %s and dependencies?"
-HDF5_INSTALL_MESSAGE = "\nHDF5 is very large and installation usually takes 5-10 minutes. Please be patient.\nIt is common to see many warnings during compilation."
+HDF5_INSTALL_MESSAGE = "\nHDF5 is very large and installation usually takes" \
+    " 5-10 minutes. Please be patient.\nIt is common to see many warnings" \
+    " during compilation."
+SETUPTOOLS_INSTALL_PROMPT = "Unable to find setuptools." \
+" It is used to download and install many of this program's prerequisites" \
+" and handle versioning. May I download and install %s?"
+
 
 # Template for cfg file contents
 CFG_FILE_TEMPLATE = """
@@ -288,7 +297,6 @@ def setup_python_home(arch_home=None):
     python_home = fix_path(prompt_user(query, default_python_home))
     make_dir(python_home)
     addsitedir(python_home)  # Load already-installed packages/eggs
-    reload(pkg_resources)  # Update working set
     return python_home, default_python_home
 
 def get_default_python_home(arch_home):
@@ -447,6 +455,33 @@ def write_pydistutils_cfg(cfg_file, arch_home,
         ofp.close()
 
 ########################### GET VERSION ########################
+def get_egg_version(module_name, *args, **kwargs):
+    """Generic version finder for installed eggs
+
+    Temporarily removed '.' from sys.path to prevent getting confused by
+    a version of the module in the current directory (but uninstalled).
+
+    Useful for if the <module>.__version__ is a revision number
+    or does not exist.
+
+    """
+    dir = os.getcwd()
+    index = None
+    if dir in sys.path:
+        index = sys.path.index(dir)
+        del sys.path[index]
+
+    try:
+        try:
+            import pkg_resources
+            ref = pkg_resources.Requirement.parse(module_name)
+            return pkg_resources.working_set.find(ref).version
+        except (AttributeError, ImportError):
+            return None
+    finally:
+        if index is not None:
+            sys.path.insert(index, dir)
+
 def get_hdf5_version():
     """Returns HDF5 version as string or None if not found or installed
 
@@ -483,7 +518,9 @@ def get_setuptools_version():
 
 def parse_download_url(url):
     """Returns a dict of URL components"""
+    components = {}
     dir, filename = os.path.split(url)
+
     # Remove extensions
     filebase = filename
     if filebase.endswith(".gz"):
@@ -493,37 +530,49 @@ def parse_download_url(url):
     if filebase.endswith(".tgz"):
         filebase = filebase[:-4]
 
-    assert "-" in filebase
     filebase_tokens = filebase.split("-")
-    version = filebase_tokens[-1]
-    program = "-".join(filebase_tokens[:-1])
-    return {"dirname": dir,
-            "file": filename,
-            "filebase": filebase,
-            "version": version,
-            "program": program}
+    # Try to extract version from filename
+    #(Assumes form: <progname>-<version>.<ext>)
+    if "-" in filebase:
+        components["version"] = filebase_tokens[-1]
+        components["program"] = "-".join(filebase_tokens[:-1])
+    else:
+        components["program"] = filebase
+
+    components["dirname"] = dir
+    components["file"] = filename
+    components["filebase"] = filebase
+    return components
 
 def str2version(ver):  # string to version object
-    if ver.startswith("$Revision:"):
-        ver = ver.split()[1]  # Get revision number
-    return LooseVersion(ver)
+    # If setuptools installed, use its versioning; else, use distutils'
+    try:
+        import pkg_resources
+        return pkg_resources.parse_version(ver)
+    except (ImportError, NameError):
+        return LooseVersion(ver)
 
 ##################### SPECIFIC PROGRAM INSTALLERS ################
 def prompt_install_hdf5(arch_home):
     return _installer("HDF5", install_hdf5, get_hdf5_version,
-                      install_message=HDF5_INSTALL_MESSAGE, arch_home=arch_home,
-                      url=HDF5_URL)
+                      install_message=HDF5_INSTALL_MESSAGE,
+                      arch_home=arch_home, url=HDF5_URL)
 
 def prompt_install_numpy(min_version=MIN_NUMPY_VERSION):
     return _installer("Numpy", install_numpy, get_numpy_version,
                       min_version=min_version,
                       install_prompt=EASY_INSTALL_PROMPT)
 
-def install_hdf5(arch_home, *args, **kwargs):
+def prompt_install_setuptools(python_home):
+    return _installer("setuptools", install_setuptools, get_setuptools_version,
+                      url=EZ_SETUP_URL, python_home=python_home,
+                      install_prompt=SETUPTOOLS_INSTALL_PROMPT)
+
+def install_hdf5(arch_home, url=HDF5_URL, *args, **kwargs):
     hdf5_dir = prompt_install_path("HDF5", arch_home)
     make_dir(hdf5_dir)
     install_dir = install_script("HDF5", hdf5_dir, HDF5_INSTALL_SCRIPT,
-                                 url=HDF5_URL)
+                                 url=url)
     return install_dir
 
 def install_numpy(min_version=MIN_NUMPY_VERSION, *args, **kwargs):
@@ -542,6 +591,28 @@ def install_numpy(min_version=MIN_NUMPY_VERSION, *args, **kwargs):
             assert "LDFLAGS" not in os.environ
             os.environ["LDFLAGS"] = env_old
 
+def install_setuptools(python_home, url=EZ_SETUP_URL, *args, **kwargs):
+    # Download ez_setup.py
+    url_components = parse_download_url(url)
+    save_path = fix_path(os.path.join(python_home, url_components["file"]))
+    if not os.path.isfile(save_path):  # Avoid duplicate downloads
+        urlretrieve(EZ_SETUP_URL, save_path)
+
+    # Run ez_setup.py to download setuptools
+    from ez_setup import download_setuptools
+    setuptools_path = download_setuptools(to_dir=python_home, delay=0)
+
+    # Execute setuptools egg to install easy_install
+    command = ["sh", "%s" % setuptools_path, "--prefix=%s" % python_home]
+    print >>sys.stderr, ">> %s" % " ".join(command)
+    code = call(command)
+    if code != 0:
+        raise InstallationError("Installation of setuptools failed.")
+    else:
+        addsitedir(python_home)  # Load new-installed packages/eggs/site.py
+
+    return True
+
 #################### GENERIC INSTALL METHODS ###################
 def _installer(progname, install_func, version_func=None,
                install_message=None, *args, **kwargs):
@@ -635,22 +706,9 @@ def _check_install(progname, version_func, min_version=None, *args, **kwargs):
 def easy_install(progname, min_version=None):
     """Easy-installs the given program (can specifiy minimum version).
     """
-    if get_setuptools_version() == None:
-        query = ("Unable to find setuptools. This is necessary to install"
-                 " this program and many of its dependencies. May I"
-                 " download and install %s?")
-        permission = prompt_install("setuptools", install_prompt=query)
-        if permission:
-            try:
-                urlretrieve(EZ_SETUP_URL)
-                from ez_setup import use_setuptools
-                use_setuptools(delay=0)
-            except:
-                print >>sys.stderr, ("Error occurred while trying to"
-                                     " install setuptools")
-                raise InstallationError()
-        else:
-            raise InstallationError()
+    # Make sure easy_install (setuptools) is installed
+    if get_setuptools_version() is None:
+        raise InstallationError("Setuptools necessary for easy_install")
 
     cmd = ["easy_install", progname.lower()]
     if min_version is not None:  # Add version requirement
@@ -754,12 +812,14 @@ def prompt_test_pytables(*args, **kwargs):
 ########################## USER INTERACTION #######################
 def prompt_install(progname, install_prompt=None, url=None,
                    version=None, default="Y", *args, **kwargs):
+    info = None
     if version is not None:
         info = "%s %s" % (progname, version)
     elif url is not None:
         url_info = parse_download_url(url)
-        info = "%s %s" % (progname, url_info["version"])
-    else:
+        if version in url_info:
+            info = "%s %s" % (progname, url_info["version"])
+    if info is None:
         info = "%s" % progname
 
     if install_prompt is None:
@@ -786,7 +846,6 @@ def prompt_yes_no(query, default="Y"):
     while True:
         # Query user and get response
         print >>sys.stderr, "%s (Y/n) [%s] " % (query, default),
-        sys.stdin.flush()
         response = raw_input().strip().lower()
         if len(response) == 0:
             response = default.strip().lower()
@@ -822,7 +881,6 @@ def prompt_user(query, default=None, choices=None):
             msg = "%s [%s] " % (prompt, default)
 
         print >>sys.stderr, msg,
-        sys.stdin.flush()
         response = raw_input().strip()
 
         if len(response) == 0:  # User didn't enter a response
@@ -881,6 +939,8 @@ def main(args=sys.argv[1:]):
         prompt_create_cfg(arch_home, python_home, default_python_home,
                           script_home, default_script_home)
 
+        prompt_install_setuptools(python_home)
+
         hdf5_dir = setup_hdf5_installation(shell, arch_home)
 
         prompt_install_numpy()
@@ -934,52 +994,16 @@ def get_R_version():
         return None
 
 def get_rpy2_version():
-    """Returns rpy2 version as a string or None if not found or installed
-
-    Temporarily removes '.' from sys.path during installation to prevent
-    finding rpy2 in current directory (but uninstalled)
-    """
-    dir = os.getcwd()
-    index = None
-    if dir in sys.path:
-        index = sys.path.index(dir)
-        del sys.path[index]
-
-    try:
-        try:
-            import rpy2
-            return rpy2.__version__
-        except (AttributeError, ImportError):
-            return None
-    finally:
-        if index is not None:
-            sys.path.insert(index, dir)
+    """Returns rpy2 version as a string or None if not found or installed"""
+    return get_egg_version("rpy2")
 
 def get_segtools_version():
     """Returns segtools version as a string or None if not found or installed
 
-    Temporarily removes '.' from sys.path during installation to prevent
-    finding segtools in current directory (but uninstalled).
-
     Since segtools __version__ is currently a revision number, get the full
     number from pkg_resources
     """
-    dir = os.getcwd()
-    index = None
-    if dir in sys.path:
-        index = sys.path.index(dir)
-        del sys.path[index]
-
-    try:
-        try:
-            ref = pkg_resources.Requirement.parse("segtools")
-            data = pkg_resources.working_set.find(ref)
-            return data.version
-        except (AttributeError, ImportError):
-            return None
-    finally:
-        if index is not None:
-            sys.path.insert(index, dir)
+    return get_egg_version("segtools")
 
 ##################### SPECIFIC PROGRAM INSTALLERS ################
 def prompt_install_rpy2():
@@ -998,7 +1022,7 @@ def prompt_install_R_libs(pkgs=R_PACKAGES, repo=CRAN_REPO):
     else:
         return False
 
-def prompt_install_segtools(min_version=MIN_SEGTOOLS_VERSION):
+def prompt_install_segtools(min_version=PKG_VERSION):
     return _installer("segtools", install_segtools, get_segtools_version,
                       install_prompt=EASY_INSTALL_PROMPT,
                       min_version=min_version)
@@ -1051,7 +1075,7 @@ def install_R_libs(pkgs=R_PACKAGES, repo=CRAN_REPO, *args, **kwargs):
         if old is not None:
             os.environ["DISPLAY"] = old
 
-def install_segtools(min_version=MIN_SEGTOOLS_VERSION, *args, **kwargs):
+def install_segtools(min_version=PKG_VERSION, *args, **kwargs):
     return easy_install("segtools", min_version=min_version)
 
 
