@@ -1,14 +1,12 @@
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 151 $"
+__version__ = "$Revision: 180 $"
 
 """
-common.py:
-
 Assorted utility functions and classes common or useful to most of segtools.
 
 
-Author: Orion Buske <orion.buske@gmail.com>
+Author: Orion Buske <stasis@uw.edu>
 """
 
 import os
@@ -20,11 +18,11 @@ from contextlib import closing, contextmanager
 from functools import partial
 from genomedata import Genome
 from gzip import open as _gzip_open
-from numpy import array, concatenate, empty, iinfo
+from numpy import array, concatenate, empty, iinfo, logical_and, uint16, uint32, int64
 from operator import itemgetter
 from pkg_resources import resource_filename, resource_string
 from string import Template
-from tabdelim import DictReader, DictWriter
+from tabdelim import DictReader, DictWriter, ListReader, ListWriter
 from rpy2.robjects import r, rinterface, numpy2ri
 # numpy2ri imported for side-effect
 
@@ -36,7 +34,8 @@ except NameError:
 PKG_R = os.extsep.join([PKG, "R"])
 PKG_RESOURCE = os.extsep.join([PKG, "resources"])
 
-from .bed import read_native
+from .bed import read_native as read_bed
+from .gff import read_native as read_gff
 
 EXT_GZ = "gz"
 EXT_PDF = "pdf"
@@ -60,9 +59,14 @@ LABEL_ALL = "all"
 
 THUMB_SIZE = 100
 
-SEGMENT_START_COL = 0
-SEGMENT_END_COL = 1
-SEGMENT_LABEL_KEY_COL = 2
+DTYPE_SEGMENT_START = int64
+DTYPE_SEGMENT_END = int64
+DTYPE_SEGMENT_KEY = uint32
+DTYPE_SOURCE_KEY = uint16
+DTYPE_STRAND = '|S1'
+MAXLEN_ID_DTYPE = 20
+DTYPE_GENE_ID = '|S%d' % MAXLEN_ID_DTYPE
+DTYPE_TRANSCRIPT_ID = '|S%d' % MAXLEN_ID_DTYPE
 
 #FEATURE_FIELDS = ["chrom", "source", "name", "start", "end", "score", "strand"]
 
@@ -75,24 +79,51 @@ class Segmentation(object):
     """
     chromosomes: a dict
       key: chromosome name
-      val: segments: numpy.ndarray(each row = chromStart, chromEnd, label_key)
-           sorted by chromStart
+      val: segments: numpy.ndarray, (each row is a (start, end, key) struct)
+           sorted by start, end
     labels: a dict
-      key: int ("label_key")  (a unique id)
+      key: int ("label_key")  (a unique id; segment['end'])
       val: printable (str or int)  (what's in the actual BED file)
 
     tracks: a list of track names that were used to obtain the segmentation
     segtool: the tool that was used to obtain this segmentation (e.g. segway)
+    name: the filename of the se
     """
 
-    def __init__(self, chromosomes, labels, tracks, segtool):
+    def __init__(self, chromosomes, labels, tracks, segtool, name):
         self.chromosomes = chromosomes
         self.labels = labels
         self.tracks = tracks
         self.segtool = segtool
+        self.name = name
 
+## Wrapper for gff/gtf feature data
+class Features(object):
+    """
+    sequences: a dict
+      key: chromosome/scaffold name
+      val: segments: numpy.ndarray
+           each row is a struct of: start, end, key, source_key
+           sorted by start, end
+    features: a dict
+      key: int ("feature_key")  (a unique id)
+      val: printable (str or int)  (what's in the actual GFF/GTF file)
+    sources: a dict
+      key: int ("source_key")  (a unique id)
+      val: printable (str or int)  (what's in the actual GFF/GTF file)
 
-class OutputMasker:
+    tracks: a list of track names that were used to obtain the segmentation
+    segtool: the tool that was used to obtain this segmentation (e.g. segway)
+    """
+    def __init__(self, sequences, features, sources):
+        self.sequences = sequences
+        self.features = features
+        self.sources = sources
+        # Allow Segmentation-like access
+        self.chromosomes = self.sequences
+        self.labels = self.features
+
+class OutputMasker(object):
     """Class to mask the output of a stream.
 
     Suggested usage:
@@ -109,19 +140,62 @@ class OutputMasker:
     def restore(self):
         return self._stream
 
-## Wrapper for gff/gtf feature data
-# class Feature(object):
-#     def __init__(self, line=None, tokens=None):
-#         assert (line or tokens) and not (line and tokens)
-#         if tokens is None:
-#             tokens = line.strip().split("\t")
+class ProgressBar(object):
+    def __init__(self, total=67, width=67, chr_done="#", chr_undone="-",
+                 format_str="Progress: [%(done)s%(undone)s]",
+                 out=sys.stdout):
+        """Create a progress bar for num_items that is width characters wide
 
-#         self.__dict__ = dict(zip(FEATURE_FIELDS, tokens))
+        total: the number of items in the task (calls to next before 100%)
+        width: the width of the bar itself, not including labels
+        chr_done: character for displaying completed work
+        chr_undone: character for displaying uncompleted work
+        format_str: format string for displaying progress bar. Must contain
+          a reference to %(done)s and %(undone)s, which will be substituted
+        out: an object that supports calls to write() and flush()
+        """
+        assert int(width) > 0
+        assert int(total) > 0
 
-#         # Make zero-based, exclusive end:
-#         #   http://genome.ucsc.edu/FAQ/FAQformat#format3
-#         self.start = int(self.start) - 1
-#         self.end = int(self.end)
+        self._width = int(width)
+        self._n = int(total)
+        self._quantum = float(self._n / self._width)
+        self._chr_done = str(chr_done)
+        self._chr_undone = str(chr_undone)
+        self._format_str = str(format_str)
+        self._out = out
+
+        self._i = 0
+        self._progress = 0
+        self.refresh()
+
+    def next(self):
+        """Advance to the next item in the task
+
+        Might or might not refresh the progress bar
+        """
+        self._i += 1
+        if self._i > self._n:
+            raise StopIteration("End of progress bar reached.")
+
+        progress = int(self._i / self._quantum)
+        if progress != self._progress:
+            self._progress = progress
+            self.refresh()
+
+    def refresh(self):
+        """Refresh the progress bar display"""
+        fields = {"done": self._chr_done * self._progress,
+                  "undone": self._chr_undone * (self._width - self._progress)}
+        self._out.write("\r%s" % (self._format_str % fields))
+        self._out.flush()
+
+    def end(self):
+        """Complete the job progress, regardless of current state"""
+        self._progress = self._width
+        self.refresh()
+        self._out.write("\n")
+        self._out.flush()
 
 
 ## UTILITY FUNCTIONS
@@ -131,6 +205,14 @@ def die(msg="Unexpected error"):
     print >> sys.stderr, "\nERROR: %s\n" % msg
     sys.exit(1)
 
+def inverse_dict(d):
+    """Given a dict, returns the inverse of the dict (val -> key)"""
+    res = {}
+    for k, v in d.iteritems():
+        assert v not in res
+        res[v] = k
+    return res
+
 def make_filename(dirpath, basename, ext):
     return os.path.join(dirpath, os.extsep.join([basename, ext]))
 
@@ -152,8 +234,8 @@ def make_id(modulename, dirpath):
 
 def check_clobber(filename, clobber):
     if (not clobber) and os.path.isfile(filename):
-        die("Output file: %s already exists! Use --clobber to overwrite!" % \
-                filename)
+        raise IOError("Output file: %s already exists!"
+                      " Use --clobber to overwrite!" % filename)
 
 def gzip_open(*args, **kwargs):
     return closing(_gzip_open(*args, **kwargs))
@@ -194,10 +276,16 @@ def template_substitute(filename):
     return Template(template_string(filename)).substitute
 
 @contextmanager
-def tab_saver(dirpath, basename, fieldnames, comment=None,
-              clobber=False, header=True, verbose=True):
-    """
-    Saves data to tab file (yeilds DictWriter)
+def tab_saver(dirpath, basename, fieldnames=None, header=True,
+              clobber=False, metadata=None, verbose=True):
+    """Save data to tab file
+
+    If fieldnames are specified, a DictWriter is yielded
+    If fieldnames are not, a ListWriter is yielded instead
+
+    metadata: a dict describing the data to include in the comment line.
+      Comment line will start with '# ' and then will be a space-delimited
+      list of <field>=<value> pairs, one for each element of the dict.
     """
     if verbose:
         print >>sys.stderr, "Saving tab file...",
@@ -205,14 +293,53 @@ def tab_saver(dirpath, basename, fieldnames, comment=None,
     outfilename = make_tabfilename(dirpath, basename)
     check_clobber(outfilename, clobber)
     with open(outfilename, "w") as outfile:
-        if comment is not None and len(comment) > 0:
-            if comment != str(comment):  # Not a single string
-                for line in comment:
-                    print >> outfile, "# %s" % line
-            else:
-                print >> outfile, "# %s" % comment
-        yield DictWriter(outfile, fieldnames,
-                         extrasaction="ignore", header=header)
+        if metadata is not None:
+            assert isinstance(metadata, dict)
+            metadata_strs = ["%s=%s" % pair for pair in metadata.iteritems()]
+            print >>outfile, "# %s" % " ".join(metadata_strs)
+
+        if fieldnames:
+            yield DictWriter(outfile, fieldnames, header=header,
+                             extrasaction="ignore")
+        else:
+            yield ListWriter(outfile)
+
+    if verbose:
+        print >>sys.stderr, "done"
+
+@contextmanager
+def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
+    """Reads data from a tab file
+
+    Yields a tuple (Reader, metadata)
+    If fieldnames is True, a DictReader is yielded
+    If fieldnames is False, a ListReader is yielded instead
+
+    metadata: a dict describing the data included in the comment line.
+    """
+    if verbose:
+        print >>sys.stderr, "Reading tab file...",
+
+    infilename = make_tabfilename(dirpath, basename)
+    if not os.path.isfile(infilename):
+        raise IOError("Unable to find tab file: %s" % infilename)
+
+    with open(infilename, "rU") as infile:
+        infile_start = infile.tell()
+        comments = infile.readline().strip().split()
+        metadata = {}
+        if comments and comments[0] == "#":
+            # Found comment line
+            for comment in comments[1:]:
+                name, value = comment.split("=")
+                metadata[name] = value
+        else:
+            infile.seek(infile_start)
+
+        if fieldnames:
+            yield DictReader(infile), metadata
+        else:
+            yield ListReader(infile), metadata
 
     if verbose:
         print >>sys.stderr, "done"
@@ -284,7 +411,7 @@ def map_segments(segments, labels, chrom_size):
           segment labeled 1 at position [4,7) gets converted into:
           0000111
     """
-    segments_dtype = segments.dtype  # MA: the data type of segments
+    segments_dtype = segments['key'].dtype  # MA: the data type of segments
     segments_dtype_max = iinfo(segments_dtype).max  # sentinel value
     # MA: the maximum value supported by the type
     assert segments_dtype_max not in labels.keys()
@@ -292,99 +419,121 @@ def map_segments(segments, labels, chrom_size):
     res = fill_array(segments_dtype_max, (chrom_size,), segments_dtype)
 
     # will overwrite in overlapping case
-    for segment in segments:
-        res[segment[SEGMENT_START_COL]:
-                segment[SEGMENT_END_COL]] = segment[SEGMENT_LABEL_KEY_COL]
+    for start, end, key in segments:
+        res[start:end] = key
+
+    return res
+
+def map_segment_label(segments, range):
+    """Flatten segments to a segment label per base vector
+
+    Returns a numpy.array of the segment key at every base
+
+    The given range must either be a tuple of (start, end), or
+      have 'start' and 'end' attributes, and the returned array
+      will represent the segment keys found between start and end.
+
+    Thus, if start is 1000, and a segment with key 4 starts at 1000, the
+    value of keys[0] is 4.
+
+    """
+    if isinstance(range, tuple):
+        map_start, map_end = range
+    else:
+        map_start = range.start
+        map_end = range.end
+
+    map_size = map_end - map_start
+    assert map_size >= 0
+
+    # Choose sentinal value as maximum value supported by datatype
+    segments_dtype = segments['key'].dtype
+    sentinal = iinfo(segments_dtype).max
+    assert sentinal != segments['key'].max()  # not already used
+
+    res = fill_array(sentinal, (map_size,), segments_dtype)
+    if map_size == 0:
+        return res
+
+    # will overwrite in overlapping case
+    for start, end, key in segments:
+        if start < map_end and end > map_start:
+            start_i = max(start - map_start, 0)
+            end_i = min(end - map_start, map_size)
+            res[start_i:end_i] = key
 
     return res
 
 ## Yields segment and the continuous corresponding to it, for each segment
 ##   in the chromosome inside of a supercontig
-def loop_segments_continuous(chromosome, segmentation, verbose=True):
+def iter_segments_continuous(chromosome, segmentation, verbose=True):
+    chrom = chromosome.name
     try:
-        segments = segmentation.chromosomes[chromosome.name]
+        segments = segmentation.chromosomes[chrom]
     except KeyError:
-        if verbose:
-            print >>sys.stderr, "\t\tskipping: no data"
         raise StopIteration
 
     supercontig_iter = chromosome.itercontinuous()
     supercontig = None
     supercontig_last_start = 0
-    segment_i = 0
-    num_segments = len(segments)
+    nsegments = len(segments)
+
+    if verbose:
+        format_str = "".join([chrom, ":\t[%(done)s%(undone)s]"])
+        progress = ProgressBar(nsegments, out=sys.stderr,
+                               format_str=format_str)
+
     for segment in segments:
-        segment_i += 1
-        seg_start = segment[SEGMENT_START_COL]
-        seg_end = segment[SEGMENT_END_COL]  # Exclusive
+        start = segment['start']
+        end = segment['end']
+
+        while supercontig is None or start >= supercontig.end:
+            try:
+                # Raise StopIteration if out of supercontigs
+                supercontig, continuous = supercontig_iter.next()
+            except StopIteration:
+                if verbose:
+                    progress.end()
+                raise
 
-        while supercontig is None or seg_start >= supercontig.end:
-            # Raise StopIteration if out of supercontigs
-            supercontig, continuous = supercontig_iter.next()
             # Enforce increasing supercontig indices
             assert supercontig.start >= supercontig_last_start
             supercontig_last_start = supercontig.start
 
-            if verbose:
-                print >>sys.stderr, "\n\t\t%s) %d : %d" % (supercontig.name,
-                                             supercontig.start,
-                                             supercontig.end)
+        if verbose:
+            progress.next()
 
-        if verbose and segment_i % 100 == 0:
-            print >>sys.stderr, "\r\t\t\tSegment %d / %d" % \
-                (segment_i, num_segments),
-            sys.stdout.flush()
-
-        if seg_end <= supercontig.start:
+        if end <= supercontig.start:
             continue  # Get next segment
 
-        yield segment, continuous[seg_start:seg_end]
+        try:
+            sc_start = supercontig.project(max(start, supercontig.start))
+            sc_end = supercontig.project(min(end, supercontig.end))
+            yield segment, continuous[sc_start:sc_end]
+        except:
+            for k, v in locals():
+                print >>sys.stderr, "%r: %r" % (k, v)
+            raise
 
     if verbose:
-        print >>sys.stderr, "\r\t\t\tSegment %d / %d" % \
-            (segment_i, num_segments)
+        progress.end()
 
+## Yields supercontig and the subset of segments which overlap it.
+def iter_supercontig_segments(chromosome, segmentation, verbose=True):
+    try:
+        segments = segmentation.chromosomes[chromosome.name]
+    except KeyError:
+        raise StopIteration
 
-## Returns splice of continuous or sequece from supercontig within segment
-## range. Datatype should be "continuous" or "sequence".
-## Assumes supercontigs are non-overlapping and sorted by start ascending
-## Returns empty array if no supercontig touches the given range
-def get_supercontig_splice(chromosome, start, end, datatype="continuous"):
-    if datatype not in ("continuous", "sequence"):
-        die("Unknown data type in 'get_supercontig_splice!")
+    for supercontig in chromosome:
+        if verbose:
+            print >>sys.stderr, "\t\t%s" % supercontig
 
-    # iterate through each supercontig
-    saved = array([])
-    for supercontig, continuous in chromosome.itercontinuous():
-        #print >>sys.stderr, "Examining supercontig [%d, %d]" % \
-        #    (supercontig.start, supercontig.end)
-
-        # Select appropriate data stream
-        if datatype == "continuous":
-            data = continuous
-        else:
-            data = supercontig.seq
-
-        # XXX Make sure there are no OBOBS in indexing!
-        if start >= supercontig.start and end <= supercontig.end:
-            # Found supercontig that completely covers range
-            return data[start - supercontig.start : end - supercontig.start]
-        elif start >= supercontig.start and start < supercontig.end:
-            # Overlapping supercontig end
-            saved = array(data[start - supercontig.start : len(data) - 1])
-        elif end > supercontig.start and end < supercontig.end:
-            # Overlapping supercontig start
-            data_splice = data[0 : end - supercontig.start]
-            # Because supercontigs in sorted order, we can return here
-            if saved.shape[0] > 0:
-                return concatenate((saved, data_splice))
-            else:
-                return data_splice
-        elif end < supercontig.start:
-            # We've passed the appropriate range
-            break
-
-    return saved
+        rows = logical_and(segments['start'] < supercontig.end,
+                           segments['end'] > supercontig.start)
+        cur_segments = segments[rows]
+        if cur_segments.shape[0] > 0:
+            yield supercontig, segments[rows]
 
 ## Ensures output directory exists and has appropriate permissions
 def setup_directory(dirname):
@@ -441,7 +590,6 @@ def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
 
     label_order, label_data = load_mnemonics(mnemonicfilename)
     str_labels = labels.values()
-
     mnemonics = []
     # Add mapping for labels in mnemonic file
     for old_label in label_order:
@@ -556,67 +704,213 @@ def load_gff_data(gff_filename, sort=True):
 
     return data
 
-def load_segmentation(filename, checknames=True):
-    """Returns a segmentation object derived from the given BED file
+def gff2arraydict(filename, gtf=False, sort=None, verbose=True):
+    """Parses a gff/gtf file and returns a dict of numpy arrays for each chrom.
 
-    If the labels in the BED file are all integers, the label_keys will
-      be integer representations of those labels (unless checknames is False).
-    If all of the labels are non-integers, the label_keys will be
-      integers corresponding to the order observed.
-    If there are some integer and some string labels, an error will be raised
-      (unless checknames is False).
+    Default behavior is to sort features unless 'gtf' is True
+
+    Returns labels, sequences, where:
+      labels: a dict from feature_key -> feature string
+      sequences: a dict from seqname -> features (numpy array)
+        (features: structured array with start, end, key, and source fields.
+         If stranded, a strand field is included.
+         If gtf-mode, gene_id and transcript_id fields are included.)
     """
-
-    print >>sys.stderr, "Loading segmentation...",
-    sys.stdout.flush()
-
-    # first get the tracks that were used for this segmentation
-    segtool, tracks = get_bed_metadata(filename)
+    if sort is None:
+        sort = not gtf
 
     data = defaultdict(list)  # A dictionary-like object
-    label_dict = {}
-    # read in as lists of tuples
+    feature_dict = {}
+    source_dict = {}
+    last_segment_start = {}  # per seq
+    unsorted_seqs = set()
+    stranded = None
     with maybe_gzip_open(filename) as infile:
-        for datum in read_native(infile):
-            label = str(datum.name)
+        for datum in read_gff(infile, gtf=gtf):
+            try:
+                if datum.start < last_segment_start[datum.seqname]:
+                    unsorted_seqs.add(datum.seqname)
+            except KeyError:
+                pass
+
+            feature = str(datum.feature)
+            try:  # Lookup feature key
+                feature_key = feature_dict[feature]
+            except KeyError:  # Map new feature to key
+                feature_key = len(feature_dict)
+                feature_dict[feature] = feature_key
+
+            source = str(datum.source)
+            try:  # Lookup source key
+                source_key = source_dict[source]
+            except KeyError:  # Map new source to key
+                source_key = len(source_dict)
+                source_dict[source] = source_key
+
+            try:
+                strand = datum.strand
+            except AttributeError:
+                strand = "."
+
+            if strand == "+" or strand == "-":
+                assert stranded is None or stranded
+                stranded = True
+            else:
+                assert not stranded
+                stranded = False
+
+            feature = [datum.start, datum.end, feature_key, source_key]
+            if stranded:
+                feature.append(datum.strand)
+
+            if gtf:
+                gene_id = datum.attributes["gene_id"]
+                transcript_id = datum.attributes["transcript_id"]
+                if len(gene_id) > MAXLEN_ID_DTYPE:
+                    raise ValueError("gene_id field was too long (over \
+%d characters): %s" % (MAXLEN_ID_DTYPE, gene_id))
+                elif len(transcript_id) > MAXLEN_ID_DTYPE:
+                    raise ValueError("transcript_id field was too long (over \
+%d characters): %s" % (MAXLEN_ID_DTYPE, transcript_id))
+                feature.extend([gene_id, transcript_id])
+
+            data[datum.seqname].append(tuple(feature))
+            last_segment_start[datum.seqname] = datum.start
+
+    # Create reverse dict for return
+    features = inverse_dict(feature_dict)
+    sources = inverse_dict(source_dict)
+
+    # convert lists of tuples to array
+    dtype = [('start', DTYPE_SEGMENT_START),
+             ('end', DTYPE_SEGMENT_END),
+             ('key', DTYPE_SEGMENT_KEY),
+             ('source_key', DTYPE_SOURCE_KEY)]
+    if stranded:
+        dtype.append(('strand', DTYPE_STRAND))
+    if gtf:
+        dtype.extend([('gene_id', DTYPE_GENE_ID),
+                      ('transcript_id', DTYPE_TRANSCRIPT_ID)])
+
+    sequences = dict((seq, array(segments, dtype=dtype))
+                       for seq, segments in data.iteritems())
+
+    # Sort segments within each chromosome
+    for seq, segments in sequences.iteritems():
+        if sort and seq in unsorted_seqs:
+            if verbose:
+                print >>sys.stderr, \
+                    "Segments were unsorted relative to %s. Sorting..." % seq,
+            segments.sort(order=['start'])
+            if verbose:
+                print >>sys.stderr, "done"
+
+    return features, sources, sequences
+
+def load_features(filename, verbose=True, *args, **kwargs):
+    if verbose:
+        print >>sys.stderr, "Loading features from: %s..." % filename
+
+    features, sources, sequences = \
+        gff2arraydict(filename, verbose=verbose, *args, **kwargs)
+
+    if verbose:
+        print >>sys.stderr, "\tFound %d sources:" % len(sources)
+        for source_key, source in sources.iteritems():
+            print >>sys.stderr, '\t\t"%s" -> %d' % (source, source_key)
+
+        print >>sys.stderr, "\tFound %d feature classes" % len(features)
+        for key, feature in features.iteritems():
+            print >>sys.stderr, '\t\t"%s" -> %d' % (feature, key)
+
+        print >>sys.stderr, "done"
+
+    return Features(sequences, features, sources)
+
+def bed2arraydict(filename, verbose=True):
+    """Parses a bedfile and returns a dict of numpy arrays for each chrom.
+
+    Returns labels, chromosomes, where:
+      labels: a dict from label_key -> label string
+      chromosomes: a dict from chrom_name -> segments (numpy array)
+        (segments: structured array with start, end, and key fields)
+
+    If the file is in BED3 format, labels will be None
+    """
+    data = defaultdict(list)  # A dictionary-like object
+    label_dict = {}
+    last_segment_start = {}  # per chromosome
+    unsorted_chroms = set()
+    with maybe_gzip_open(filename) as infile:
+        for datum in read_bed(infile):
+            try:
+                if datum.chromStart < last_segment_start[datum.chrom]:
+                    unsorted_chroms.add(datum.chrom)
+            except KeyError:
+                pass
+
+            try:
+                label = str(datum.name)
+            except AttributeError:
+                # No name column, read as BED3 (no labels)
+                label = ""
+
             try:  # Lookup label key
                 label_key = label_dict[label]
             except KeyError:  # Map new label to key
-                if checknames:
-                    try:
-                        label_key = int(label)
-                    except ValueError:
-                        checknames = False
-                        if len(label_dict) > 0:  # Previous int labels found
-                            die("Found both integer and string labels in BED"
-                                "file: %s" % filename)
-                if not checknames:  # Sequential if's important
-                    label_key = len(label_dict)
-
+                label_key = len(label_dict)
                 label_dict[label] = label_key
 
             segment = (datum.chromStart, datum.chromEnd, label_key)
             data[datum.chrom].append(segment)
+            last_segment_start[datum.chrom] = datum.chromStart
 
     # Create reverse dict for return
-    labels = dict((val, key) for key, val in label_dict.iteritems())
-    print >>sys.stderr, "\nMapping names to integers"
-    for key, label in labels.iteritems():
-        print >>sys.stderr, "\"%s\" -> %d" % (label, key)
-
-    # Sort segments within each chromosome by start index
-    for segments in data.itervalues():
-        segments.sort(key=itemgetter(0))
+    labels = inverse_dict(label_dict)
 
     # convert lists of tuples to array
-    chromosomes = dict((chrom, array(segments))
+    dtype = [('start', DTYPE_SEGMENT_START),
+             ('end', DTYPE_SEGMENT_END),
+             ('key', DTYPE_SEGMENT_KEY)]
+    chromosomes = dict((chrom, array(segments, dtype=dtype))
                        for chrom, segments in data.iteritems())
 
-    print >>sys.stderr, "done."
-    # wrap in a Segmentation object
-    return Segmentation(chromosomes, labels, tracks, segtool)
+    # Sort segments within each chromosome
+    for chrom, segments in chromosomes.iteritems():
+        if chrom in unsorted_chroms:
+            if verbose:
+                print >>sys.stderr, "Segments were unsorted relative to \
+%s. Sorting..." % chrom,
+            segments.sort(order=['start'])
+            if verbose:
+                print >>sys.stderr, "done"
 
-## Returns a genome object from data in a genomedata directory
+    return labels, chromosomes
+
+def load_segmentation(filename, verbose=True):
+    """Returns a segmentation object derived from the given BED4+ file
+
+    label_keys are integers corresponding to the order observed.
+    """
+
+    if verbose:
+        print >>sys.stderr, "Loading segmentation...",
+
+    # first get the tracks that were used for this segmentation
+    segtool, tracks = get_bed_metadata(filename)
+
+    labels, chromosomes = bed2arraydict(filename)
+
+    if verbose:
+#         print >>sys.stderr, "\n\tMapped labels to integer keys:"
+#         for key, label in labels.iteritems():
+#             print >>sys.stderr, "\t\t\"%s\" -> %d" % (label, key)
+        print >>sys.stderr, "done."
+
+    # wrap in a Segmentation object
+    return Segmentation(chromosomes, labels, tracks, segtool, filename)
+
+## Returns a Genome object from data in a genomedata directory
 def load_genome(genomedatadir):
     if genomedatadir is not None and os.path.isdir(genomedatadir):
         genome = Genome(genomedatadir)
