@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 151 $"
+__version__ = "$Revision: 178 $"
 
 """
 validate.py:
@@ -19,11 +19,7 @@ from collections import defaultdict
 from numpy import zeros
 from rpy2.robjects import r, numpy2ri
 
-# XXX: Do this without the kludgy constants
-from .common import die, get_ordered_labels, get_supercontig_splice, \
-    image_saver, load_segmentation, load_genome, make_namebase_summary, \
-    make_tabfilename, map_mnemonics, r_source, SEGMENT_START_COL, \
-    SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver
+from .common import die, get_ordered_labels, image_saver, load_segmentation, load_genome, make_namebase_summary, make_tabfilename, map_mnemonics, r_source, setup_directory, tab_saver
 
 from .html import save_html_div
 
@@ -79,12 +75,12 @@ def calc_nucleotide_frequencies(segmentation, genome,
         for entry in category:
             #print("Mapping %s%s -> %d" % (entry[0], entry[1], index))
             quick_dinuc_categories[entry] = index
-    
+
     # Store counts of each (di)nucleotide observed
     # separated by label
     nuc_counts = defaultdict(dict)
     dinuc_counts = defaultdict(dict)
-    
+
     labels = segmentation.labels
     for label_key in labels.iterkeys():
         nuc_counts[label_key] = zeros(len(nuc_categories)+1, dtype=numpy.long)
@@ -99,45 +95,39 @@ def calc_nucleotide_frequencies(segmentation, genome,
         try:
             segments = segmentation.chromosomes[chrom]
         except KeyError:
-            print >>sys.stderr, "\t\tskipping: no data"
             continue
 
-        for segment in segments:                    
-            seg_start = segment[SEGMENT_START_COL]   
-            seg_end = segment[SEGMENT_END_COL]
-            seg_label = segment[SEGMENT_LABEL_KEY_COL]
+        for segment in segments:
+            seg_start, seg_end, seg_label = segment
 
-            sequence = get_supercontig_splice(chromosome, seg_start,
-                                              seg_end, "sequence")
+            sequence = chromosome.seq[seg_start:seg_end]
+            sequence = sequence.tostring().upper()
 
-            if sequence.shape[0] > 0:
-                # XXXopt: could be optimized to use matrix operations
-                # to determine number of occurances of each (di)nuc
-                
-                # Inch through iteration to look at pairs efficiently
-                cur_nuc = None
-                prev_nuc = None
-                for nuc in sequence:
-                    prev_nuc = cur_nuc
-                    cur_nuc = chr(nuc).upper()  # Get as uppercase letter
-                        
-                    # Record nucleotide
-                    nuc_cat = quick_nuc_categories[cur_nuc]
-                    if nuc_cat == {}:  # Didn't find; put in last bin
-                        #print >>sys.stderr, "Unknown nuc: %s" % (cur_nuc)
-                        nuc_cat = len(nuc_counts[seg_label]) - 1
-                        
-                    nuc_counts[seg_label][nuc_cat] += 1
+            # XXXopt: could be optimized to use matrix operations
+            # to determine number of occurances of each (di)nuc
+            # Inch through iteration to look at pairs efficiently
+            cur_nuc = None
+            prev_nuc = None
+            for nuc in sequence:
+                prev_nuc = cur_nuc
+                cur_nuc = nuc
 
-                    # Record dinucleotide (last and current)
-                    if prev_nuc is not None:
-                        dinuc_cat = quick_dinuc_categories[(prev_nuc, cur_nuc)]
-                        if dinuc_cat == {}:  # Didn't find; put in last bin
-                            #print >>sys.stderr, "Unknown dinuc: %s" % [(prev_nuc, cur_nuc)]
-                            dinuc_cat = len(dinuc_counts[seg_label]) - 1
+                # Record nucleotide
+                nuc_cat = quick_nuc_categories[cur_nuc]
+                if nuc_cat == {}:  # Didn't find; put in last bin
+                    #print >>sys.stderr, "Unknown nuc: %s" % (cur_nuc)
+                    nuc_cat = len(nuc_counts[seg_label]) - 1
+
+                nuc_counts[seg_label][nuc_cat] += 1
+
+                # Record dinucleotide (last and current)
+                if prev_nuc is not None:
+                    dinuc_cat = quick_dinuc_categories[(prev_nuc, cur_nuc)]
+                    if dinuc_cat == {}:  # Didn't find; put in last bin
+                        dinuc_cat = len(dinuc_counts[seg_label]) - 1
+
+                    dinuc_counts[seg_label][dinuc_cat] += 1
 
-                        dinuc_counts[seg_label][dinuc_cat] += 1
-                
         # Only look at first chromosome if quick
         if quick: break
 
@@ -146,9 +136,9 @@ def calc_nucleotide_frequencies(segmentation, genome,
 
 def make_row(label, nuc_counts, dinuc_counts, fieldnames=FIELDNAMES):
     row = {}
-    
+
     for i, field in enumerate(fieldnames):
-        if field == "label": 
+        if field == "label":
             row[field] = label
         elif i <= len(nuc_counts):
             # Compensate for "label" field
@@ -175,14 +165,15 @@ def make_summary_row(label, nuc_counts, dinuc_counts, fieldnames=FIELDNAMES):
     nuc_sum = nuc_counts.sum()
     dinuc_sum = dinuc_counts.sum()
     for i, field in enumerate(fieldnames):
-        if field == "label": 
+        if field == "label":
             row[field] = label
         elif i <= len(nuc_counts):
             # Compensate for "label" field
             row[field] = "%.3f" % (nuc_counts[i - 1] / nuc_sum)
         else:
             # Compensate for "label" and nucleotide fields
-            row[field] = "%.3f" % (dinuc_counts[i - len(nuc_counts) - 1] / dinuc_sum)
+            row[field] = "%.3f" % (dinuc_counts[i - len(nuc_counts) - 1] / \
+                                       dinuc_sum)
     return row
 
 # Fieldnames must agree with categories in order and content
@@ -204,12 +195,12 @@ def save_plot(dirpath, clobber=False, mnemonics=[]):
     tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
-        
+
     # Plot data in file
     with image_saver(dirpath, NAMEBASE, clobber=clobber,
                      width=PNG_SIZE, height=PNG_SIZE):
         r.plot(r["plot.dinuc"](tabfilename, mnemonics = mnemonics))
-    
+
 def save_html(dirpath, clobber=False):
     save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                   tablenamebase=NAMEBASE_SUMMARY, module=MODULE,
@@ -221,13 +212,13 @@ def validate(bedfilename, genomedatadir, dirpath, clobber=False, quick=False,
     setup_directory(dirpath)
     segmentation = load_segmentation(bedfilename)
     genome = load_genome(genomedatadir)
-    
+
     assert segmentation is not None
     assert genome is not None
 
     labels = segmentation.labels
     mnemonics = map_mnemonics(labels, mnemonicfilename)
-    
+
     if not replot:
         nuc_counts, dinuc_counts = calc_nucleotide_frequencies(
             segmentation, genome, quick=quick)
@@ -239,26 +230,26 @@ def validate(bedfilename, genomedatadir, dirpath, clobber=False, quick=False,
         save_plot(dirpath, clobber=clobber, mnemonics=mnemonics)
 
     save_html(dirpath, clobber=clobber)
-    
+
 def parse_options(args):
     from optparse import OptionParser
 
     usage = "%prog [OPTIONS] BEDFILE GENOMEDATADIR"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
-    
+
     parser.add_option("--clobber", action="store_true",
                       dest="clobber", default=False,
                       help="Overwrite existing output files if the specified"
                       " directory already exists.")
-    parser.add_option("--quick", action="store_true", 
+    parser.add_option("--quick", action="store_true",
                       dest="quick", default=False,
                       help="Compute values only for one chromosome.")
-    parser.add_option("--replot", action="store_true", 
+    parser.add_option("--replot", action="store_true",
                       dest="replot", default=False,
                       help="Load data from output tab files and"
                       " regenerate plots instead of recomputing data")
-    parser.add_option("--noplot", action="store_true", 
+    parser.add_option("--noplot", action="store_true",
                       dest="noplot", default=False,
                       help="Do not generate plots")
 
@@ -266,7 +257,7 @@ def parse_options(args):
                       default=None,
                       help="If specified, labels will be shown using"
                       " mnemonics found in this file")
-    parser.add_option("-o", "--outdir", 
+    parser.add_option("-o", "--outdir",
                       dest="outdir", default="%s" % MODULE,
                       help="File output directory (will be created"
                       " if it does not exist) [default: %default]")
@@ -275,12 +266,12 @@ def parse_options(args):
 
     if len(args) < 2:
         parser.error("Insufficient number of arguments")
-        
+
     if options.noplot and options.replot:
         parser.error("noplot and replot are contradictory")
 
     return (options, args)
-    
+
 ## Command-line entry point
 def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
@@ -291,6 +282,6 @@ def main(args=sys.argv[1:]):
              clobber=options.clobber, quick=options.quick,
              replot=options.replot, noplot=options.noplot,
              mnemonicfilename=options.mnemonicfilename)
-        
+
 if __name__ == "__main__":
     sys.exit(main())
